/*****************************************************************************
 UNIVERSIDAD DE ZARAGOZA
 DEPARTAMENTO DE INFORMATICA E INGENIERIA DE SISTEMAS
 AREA DE LENGUAJES Y SISTEMAS INFORMATICOS
******************************************************************************
******************************************************************************
 Fichero     :  clases.c
 Asignatura  :  Vision por Computador
******************************************************************************/

/*****************************************************************************
                              I N C L U D E S
******************************************************************************/

#include <stdio.h>
#include <math.h>
#include "clases.h"

/*****************************************************************************
                            C O N S T A N T E S  
******************************************************************************/

#define FALSE     0
#define TRUE      1
#define NULO      -1

/*****************************************************************************
                              F U N C I O N E S
******************************************************************************/

/*****************************************************************************/
void leer_clases (int ndescriptores)

/*****************************************************************************
******************************************************************************/
{
  int clase, descriptor;
  FILE *f;

  if ((f = fopen ("clases.dat", "r")) == NULL)
  {
    fprintf (stderr, "Fichero clases no existe.\n");
    exit (1);
  }

  fscanf (f, "%d\n", &(NCLASES));
  for (clase = 0; clase < clases.clases; ++clase)
  {
      fscanf (f, "%s\n", &(NOMBRE(clase)));
      fscanf (f, "%d\n", &(MUESTRAS(clase)));

      for (descriptor = 0; descriptor < ndescriptores; ++descriptor)
      {
          int muestra;

          for (muestra = 0; muestra < MUESTRAS(clase); ++muestra)
              fscanf (f, "%lg\n", &(MUESTRA(clase,descriptor,muestra)));
          fscanf (f, "%lg %lg\n", &(MEDIA(clase,descriptor)), &(VARIANZA(clase,descriptor)));
      }
  }

  fclose (f);
}

/*****************************************************************************/
void escribir_clases (int ndescriptores)

/*****************************************************************************
******************************************************************************/
{
  int clase, descriptor;
  FILE *f;

  f = fopen ("clases.dat", "w");

  fprintf (f, "%d\n", NCLASES);
  for (clase = 0; clase < NCLASES; ++clase)
  {
      fprintf (f, "%s\n", NOMBRE(clase));
      fprintf (f, "%d\n", MUESTRAS(clase));
      for (descriptor = 0; descriptor < ndescriptores; ++descriptor)
      {
          int muestra;

          for (muestra = 0; muestra < MUESTRAS(clase); ++muestra)
              fprintf (f, "%g\n", MUESTRA(clase,descriptor,muestra));
          fprintf (f, "%g %g\n", MEDIA(clase,descriptor), VARIANZA(clase,descriptor));
      }
  }

  fclose (f);
}

/*****************************************************************************/
int buscar_clase (char *nombre)

/*****************************************************************************
******************************************************************************/
{
  int clase = 0;
  int encontrado = FALSE;

  while ((clase < NCLASES) && (!encontrado))
        if (!strcasecmp(NOMBRE(clase), nombre))
           encontrado = TRUE;
        else
           ++clase;

  return (encontrado ? clase : NULO);
}

/*****************************************************************************/
int crear_clase (char *nombre)

/*****************************************************************************
******************************************************************************/
{
  int clase;

  if (NCLASES == MAXCLASES)
  {
     fprintf (stderr, "Ya no pueden crearse mas clases!!!\n");
     clase = NULO;
  }
  else
  {
     strcpy (NOMBRE(NCLASES), nombre);
     MUESTRAS(NCLASES) = 0;
     clase = NCLASES;
     ++(NCLASES);
  }

  return clase;
}

/*****************************************************************************/
void anadir_muestra (int clase, double descriptores[], int ndescriptores)

/*****************************************************************************
******************************************************************************/
{
  int muestras = MUESTRAS(clase);
  int descriptor;

  for (descriptor = 0; descriptor < ndescriptores; ++descriptor)
      MUESTRA(clase,descriptor,muestras) = descriptores[descriptor];

  ++(MUESTRAS(clase));
}

/*****************************************************************************/
void actualizar_medias_y_varianzas (int clase, int ndescriptores)

/*****************************************************************************
Suponiendo que se han anyadido nuevas muestras al clase del descriptor
(seran las ultimas del vector), se debe actualizar el valor medio y la 
varianza de cada uno de los descriptores.
******************************************************************************/
{
   int p, m;

   for (p = 0; p < ndescriptores; ++p)
   {
       MEDIA (clase, p) = 0;
       for (m = 0; m < MUESTRAS(clase); ++m)
           MEDIA (clase, p) += MUESTRA (clase, p, m);
	   
       MEDIA (clase, p) /= MUESTRAS (clase);

       if (MUESTRAS(clase) > 1)
       {
          VARIANZA (clase, p) = 0;
          for (m = 0; m < MUESTRAS(clase); ++m)
              VARIANZA (clase, p) += 
			 (MEDIA (clase, p) - MUESTRA (clase, p, m))*
			 (MEDIA (clase, p) - MUESTRA (clase, p, m));
	   
          VARIANZA (clase, p) /= (MUESTRAS (clase) - 1);
       }
       else
          VARIANZA (clase, p) = 0;
   }
}

/*****************************************************************************/
void ordenar_clases ()

/*****************************************************************************
******************************************************************************/
{
  int o1, o2;

  for (o1 = 0; o1 < NCLASES; ++o1)
      for (o2 = o1 + 1; o2 < NCLASES; ++o2)
          if (strcmp (NOMBRE(o1), NOMBRE(o2)) > 0)
      {
         CLASE obj = clases.clase[o1];

         clases.clase[o1] = clases.clase[o2];
         clases.clase[o2] = obj;
      }
}

/*****************************************************************************/
void ordenar_clases_por_descriptor (int p)

/*****************************************************************************
******************************************************************************/
{
  int o1, o2;

  for (o1 = 0; o1 < NCLASES; ++o1)
      for (o2 = o1 + 1; o2 < NCLASES; ++o2)
          if (MEDIA (o1,p) > MEDIA (o2,p))
      {
         CLASE obj = clases.clase[o1];

         clases.clase[o1] = clases.clase[o2];
         clases.clase[o2] = obj;
      }
}

